#define _GNU_SOURCE
#include "workspace.hh"
#include "window.hh"
#include "session.hh"

#include <stdio.h>


WorkspaceMenu::WorkspaceMenu(Workspace *w, BlackboxSession *s) :
  BlackboxMenu(s) { workspace = w; }
void WorkspaceMenu::showMenu(void)
{ BlackboxMenu::showMenu(); }
void WorkspaceMenu::hideMenu(void)
{ BlackboxMenu::hideMenu(); }
void WorkspaceMenu::moveMenu(int x, int y)
{ BlackboxMenu::moveMenu(x, y); }
void WorkspaceMenu::updateMenu(void)
{ BlackboxMenu::updateMenu(); }
int WorkspaceMenu::insert(char **u)
{ return BlackboxMenu::insert(u); }
int WorkspaceMenu::remove(int i)
{ return BlackboxMenu::remove(i); }
void WorkspaceMenu::titlePressed(int) { }
void WorkspaceMenu::titleReleased(int button) { if (button == 3) hideMenu(); }
void WorkspaceMenu::itemPressed(int, int) { }
void WorkspaceMenu::itemReleased(int, int) { }


WorkspaceManagerMenu::WorkspaceManagerMenu(WorkspaceManager *m,
					   BlackboxSession *s) :
  BlackboxMenu(s)
{ ws_manager = m; }
void WorkspaceManagerMenu::showMenu(void)
{ BlackboxMenu::showMenu(); }
void WorkspaceManagerMenu::hideMenu(void)
{ BlackboxMenu::hideMenu(); }
void WorkspaceManagerMenu::moveMenu(int x, int y)
{ BlackboxMenu::moveMenu(x, y); }
void WorkspaceManagerMenu::updateMenu(void)
{ BlackboxMenu::updateMenu(); }
int WorkspaceManagerMenu::insert(char *l)
{ return BlackboxMenu::insert(l); }
void WorkspaceManagerMenu::titlePressed(int) { }


void WorkspaceManagerMenu::titleReleased(int button)
{ if (button == 3) hideMenu(); }


void WorkspaceManagerMenu::itemPressed(int, int) { }
void WorkspaceManagerMenu::itemReleased(int button, int item) {
  if (button == 1)
    if (item == 0) {
      Workspace *wkspc = new Workspace(ws_manager,
				       ws_manager->workspaces_list->count());
      ws_manager->workspaces_list->append(wkspc);
      insert(wkspc->name());
      updateMenu();
    } else if (item == 1) {
      int i;
      printf("total workspaces %d\n",
	     i = ws_manager->workspaces_list->count());
      if (i > 1) {
	Workspace *wkspc = ws_manager->workspaces_list->at(i - 1);
	if (ws_manager->currentWorkspaceID() == (i - 1))
	  ws_manager->changeWorkspaceID(i - 2);
	wkspc->removeAll();
	ws_manager->workspaces_list->remove(i - 1);
	BlackboxMenu::remove(i + 1);
	delete wkspc;
	updateMenu();
      }
      printf("total workspaces %d\n", ws_manager->workspaces_list->count());
    } else if ((item - 2) != ws_manager->currentWorkspaceID())
      ws_manager->changeWorkspaceID(item - 2);
}


Workspace::Workspace(WorkspaceManager *m, int id) {
  ws_manager = m;
  workspace_id = id;

  workspace_list = new llist<BlackboxWindow>;
  workspace_menu = new WorkspaceMenu(this, ws_manager->Session());

  workspace_name = new char[20];
  sprintf(workspace_name, "Workspace %d", workspace_id);
  workspace_menu->setMenuLabel(workspace_name);
}


Workspace::~Workspace(void) {
  delete workspace_list;
  delete workspace_menu;
  delete [] workspace_name;
}


void Workspace::showMenu(void) { workspace_menu->showMenu(); }
void Workspace::hideMenu(void) { workspace_menu->hideMenu(); }
void Workspace::moveMenu(int x, int y) { workspace_menu->moveMenu(x, y); }
void Workspace::updateMenu(void) { workspace_menu->updateMenu(); }


const int Workspace::addWindow(BlackboxWindow *w) {
  w->setWorkspace(workspace_id);
  w->setWindowNumber(workspace_list->count());
  workspace_list->append(w);
  workspace_menu->insert(w->Title());
  workspace_menu->updateMenu();
  return w->windowNumber();
}


const int Workspace::removeWindow(BlackboxWindow *w) {
  workspace_menu->remove((const int) w->windowNumber());
  workspace_list->remove((const int) w->windowNumber());
  for (int i = 0; i < workspace_list->count(); ++i)
    workspace_list->at(i)->setWindowNumber(i);

  workspace_menu->updateMenu();
  return workspace_list->count();
}


int Workspace::showAll(void) {
  BlackboxWindow *win;

  int i;
  for (i = 0; i < workspace_list->count(); ++i) {
    win = workspace_list->at(i);
    if (! win->isIconic())
      win->deiconifyWindow();
  }
  
  return i;
}


int Workspace::hideAll(void) {
  BlackboxWindow *win;

  int i;
  for (i = 0; i < workspace_list->count(); ++i) {
    win = workspace_list->at(i);
    if (! win->isIconic())
      win->withdrawWindow();
  }

  return i;
} 


int Workspace::removeAll(void) {
  BlackboxWindow *win;

  int i;
  for (i = 0; i < workspace_list->count(); ++i) {
    win = workspace_list->at(i);
    ws_manager->currentWorkspace()->addWindow(win);
    if (! win->isIconic())
      win->iconifyWindow();
  }

  return i;
}


void Workspace::Dissociate(void) {
  BlackboxWindow *win;
  Display *display = ws_manager->Session()->control();
  int i;

  XGrabServer(display);
  for (i = 0; i < workspace_list->count(); ++i) {
    win = workspace_list->at(i);
    win->deiconifyWindow();
    XUnmapWindow(display, win->frameWindow());
    XReparentWindow(display, win->clientWindow(),
		    ws_manager->Session()->Root(), win->XFrame(),
		    win->YFrame());
    XMoveResizeWindow(display, win->clientWindow(), win->XFrame(),
		      win->YFrame(), win->clientWidth(), win->clientHeight());
    XMapWindow(display, win->clientWindow());
  }

  XSync(display, 0);
  XUngrabServer(display);
}


WorkspaceManager::WorkspaceManager(BlackboxSession *s, int c) {
  Workspace *wkspc = 0;
  int i;
  session = s;
  
  workspaces_list = new llist<Workspace>;
  workspaces_menu = new WorkspaceManagerMenu(this, session);

  char *title = new char[11];
  sprintf(title, "Workspaces");
  workspaces_menu->setMenuLabel(title);
  workspaces_menu->insert("New Workspace");
  workspaces_menu->insert("Delete last");

  if (c != 0) {
    for (i = 0; i < c; ++i) {
      wkspc = new Workspace(this, i);
      workspaces_list->append(wkspc);
      workspaces_menu->insert(wkspc->name());
    }
  } else {
    wkspc = new Workspace(this, 0);
    workspaces_list->append(wkspc);
    workspaces_menu->insert(wkspc->name());
  }

  workspaces_menu->updateMenu();
  current = workspaces_list->at(0);
}


WorkspaceManager::~WorkspaceManager(void) {
  DissociateAll();
  delete workspaces_menu;

  for (int i = 0; i < workspaces_list->count(); ++i) {
    delete workspaces_list->at(0);
    workspaces_list->remove(0);
  }
}


void WorkspaceManager::DissociateAll(void) {
  for (int i = 0; i < workspaces_list->count(); ++i)
    workspaces_list->at(i)->Dissociate();
}


Workspace *WorkspaceManager::workspace(int w) {
  return workspaces_list->at(w);
}


void WorkspaceManager::changeWorkspaceID(int id) {
  current->hideMenu();
  current->hideAll();
  current = workspace(id);
  current->showAll();
}


void WorkspaceManager::showMenu(void) { workspaces_menu->showMenu(); }
void WorkspaceManager::hideMenu(void) { workspaces_menu->hideMenu(); }
void WorkspaceManager::moveMenu(int x, int y)
{ workspaces_menu->moveMenu(x, y); }
