#ifndef _blackbox_workspace_hh
#define _blackbox_workspace_hh

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/Xresource.h>
#include <X11/xpm.h>
#ifdef SHAPE
#include <X11/extensions/shape.h>
#endif

class BlackboxSession;
class BlackboxWindow;
class Workspace;
class WorkspaceMenu;
class WorkspaceManager;
class WorkspaceManagerMenu;

#include "menu.hh"
#include "llist.hh"


class WorkspaceMenu : public BlackboxMenu {
private:
  Workspace *workspace;


protected:
  virtual void titlePressed(int);
  virtual void titleReleased(int);
  virtual void itemPressed(int, int);
  virtual void itemReleased(int, int);


public:
  WorkspaceMenu(Workspace *, BlackboxSession *s);

  void showMenu();
  void hideMenu();
  void moveMenu(int, int);
  void updateMenu();

  int insert(char **);
  int remove(int);
};


class WorkspaceManagerMenu : public BlackboxMenu {
private:
  WorkspaceManager *ws_manager;
  

protected:
  virtual void titlePressed(int);
  virtual void titleReleased(int);
  virtual void itemPressed(int, int);
  virtual void itemReleased(int, int);


public:
  WorkspaceManagerMenu(WorkspaceManager *, BlackboxSession *);

  void showMenu();
  void hideMenu();
  void moveMenu(int, int);
  void updateMenu();

  int insert(char *);
  int remove(int);
};


class Workspace {
private:
  WorkspaceManager *ws_manager;
  llist<BlackboxWindow> *workspace_list;
  WorkspaceMenu *workspace_menu;

  char *workspace_name;
  int workspace_id;

protected:

public:
  Workspace(WorkspaceManager *, int = 0);
  ~Workspace(void);

  const int addWindow(BlackboxWindow *);
  const int removeWindow(BlackboxWindow *);
  int showAll(void);
  int hideAll(void);
  int removeAll(void);
  void Dissociate(void);
  void showMenu(void);
  void hideMenu(void);
  void moveMenu(int, int);
  void updateMenu(void);
  
  int workspaceID(void)
    { return workspace_id; }
  char *name(void)
    { return workspace_name; }
};


class WorkspaceManager {
private:
  llist<Workspace> *workspaces_list;
  WorkspaceManagerMenu *workspaces_menu;
  BlackboxSession *session;
  Workspace *current;

  friend WorkspaceManagerMenu;

protected:

public:
  WorkspaceManager(BlackboxSession *, int = 1);
  ~WorkspaceManager(void);

  int addWorkspace(void);
  int removeWorkspaceID(int);
  void changeWorkspaceID(int);
  Workspace *workspace(int);
  void DissociateAll(void);
  void showMenu(void);
  void hideMenu(void);
  void moveMenu(int, int);

  Workspace *currentWorkspace(void)
    { return current; }
  BlackboxSession *Session(void)
    { return session; }
  int count(void)
    { return workspaces_list->count(); }
  int currentWorkspaceID(void)
    { return current->workspaceID(); }
};


#endif
